#include "EasyInput.h"

char *EI_input_string(int string_length, const char *chars_allowed)
{
	char *string;
	unsigned int key = 0;
	int i = 0, j = 0;
	int displacement = 0;
	int caps = 0;
	char arrow[] = {0xE6, 0x9A};
	EI_config *config = (void*)(0);

	// Get configuration
	config = (EI_config*)EI_manage_config(EI_GET_ALL, (void*)(0));


	// Allocate RAM for string
	string = (char*)malloc(sizeof(char) * string_length);


	// Initialize string
	for(i=0; i<string_length; i++) string[i] = 0;


	// Setup cursor mode	
	if(config->start_mode == EI_ALPHA_LOCKED)
	{
		EI_KBD_PutKey(KEY_CTRL_SHIFT, 0);
		GetKey(&key);
	}

	if(config->start_mode >= EI_ALPHA)
	{
		EI_KBD_PutKey(KEY_CTRL_ALPHA, 0);
		GetKey(&key);
		EI_Cursor_SetFlashStyle(2);
	}

	EI_Cursor_SetFlashMode(1);


	// Main loop
	i = 0;
	while(key != KEY_CTRL_EXE && key != KEY_CTRL_AC)
	{
		switch(config->align) // Set cursor position
		{
			case EI_LEFT : EI_Cursor_SetPosition(config->column, config->row);
				break;

			case EI_CENTER : EI_Cursor_SetPosition((int)((2 * config->column + config->box_length - 1) / 2 - EI_str_length(string) / 2), config->row);
				break;

			case EI_RIGHT : EI_Cursor_SetPosition(config->column + ((displacement) ? 0 : config->box_length - EI_str_length(string) + 1), config->row);
				break;
		}

		if(displacement) Print((const unsigned char*)arrow); // display the arrow if some text is out of box

		Print((const unsigned char*)(string + displacement)); // Print string

		GetKey(&key); // Get getkey value

		if(key == config->Aa_key) // Change input mode
		{
			caps = 1 - caps;
		}

		if(caps && key >= 65 && key <= 90) // If we have to change case
				key += 32;

		if(i < string_length && EI_check_char(key, chars_allowed)) // Add char to string
		{
			string[i] = key;
			i++;
		}

		if(key == KEY_CTRL_DEL) // Check if last character has to be deleted
		{
			i--;
			string[i] = 0;
			EI_Cursor_SetPosition(config->column, config->row);

			for(j = 0; j<string_length; j++)
				Print((const unsigned char*)" ");
		}

		displacement = (i >= config->box_length) ? i - config->box_length : 0; // Create new cursor displacement value

		if(i < 0) i = 0;
		if(i == string_length) string[i] = 0;
		if(i > string_length) i = string_length;
	}

	// Set off cursor mode
	EI_Cursor_SetFlashMode(0);
	EI_Cursor_SetFlashStyle(0);


	// Return NULL in case of an input break
	if(key == KEY_CTRL_AC)
	{
		free(string);
		return (char*)(0);
	}


	// Return the string
	return string;
}

const void *EI_manage_config(int parameter, const void *value)
{
	static EI_config config;

	if(value)
	{
		switch(parameter)
		{
			case EI_SET_COLUMN: config.column = *((unsigned int*)value) - 1;
				break;
			case EI_SET_ROW: config.row = *((unsigned int*)value) - 1;
				break;
			case EI_SET_BOX_LENGTH: config.box_length = *((unsigned int*)value) - 2;
				break;
			case EI_SET_ALIGN: config.align = *((unsigned int*)value);
				break;
			case EI_SET_START_MODE: config.start_mode = *((unsigned int*)value);
				break;
			case EI_SET_Aa_KEY: config.Aa_key = *((unsigned int*)value);
				break;
		}
	}
	else
	{
		switch(parameter)
		{
			case EI_GET_ALL: return (const void*)(&config);
			case EI_SET_COLUMN: return (const void*)&config.column;
			case EI_SET_ROW: return (const void*)&config.row;
			case EI_SET_BOX_LENGTH: return (const void*)&config.box_length;
			case EI_SET_ALIGN: return (const void*)&config.align;
			case EI_SET_START_MODE: return (const void*)&config.start_mode;
			case EI_SET_Aa_KEY: return (const void*)&config.Aa_key;
		}
	}

	return (void*)(0);
}

void EI_init(void)
{
	unsigned int var;

	var = 1;
	EI_manage_config(EI_SET_COLUMN, &var);
	var = 1;
	EI_manage_config(EI_SET_ROW, &var);
	var = EI_LEFT;
	EI_manage_config(EI_SET_ALIGN, &var);
	var = EI_ALPHA_LOCKED;
	EI_manage_config(EI_SET_START_MODE, &var);
	var = 21;
	EI_manage_config(EI_SET_BOX_LENGTH, &var);
	var = 30013;
	EI_manage_config(EI_SET_Aa_KEY, &var);
}

int EI_str_length(const char *str)
{
	int i = 0;

	while(str[i]) i++;
	
	return i;
}

int EI_check_char(char c, const char *chars_allowed)
{
	int i = 0;

	while(chars_allowed[i])
	{
		if(c == chars_allowed[i])
			return 1;
		i++;
	}

	return 0;
}